/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#pragma once

#include "OE_document.h"
class OE_curve;

class OE_actions
{
	public:
		/** Default constructor */
		OE_actions(OE_document* doc);
		/** Default destructor */
		virtual ~OE_actions();

		/** \brief undo the action
		 * \param document -> the document we're working on
		 */
		virtual void undo() = 0;

		/** \brief redo the action
		 * \param document -> the document we're working on
		 */
		virtual void redo() = 0;

		OE_curve* switchVal(OE_pointcurve* Original, OE_pointcurve** storage);
		std::list<OE_pointcurve*> switchVal(std::list<OE_pointcurve*> Original, std::list<OE_pointcurve*>* storage);
		OE_stitchs* switchVal(OE_stitchs* Original, OE_stitchs** storage);
		std::list<OE_stitchs*> switchVal(std::list<OE_stitchs*> Original, std::list<OE_stitchs*>* storage);
		float switchVal(float Original, float* storage);
		unsigned switchVal(unsigned Original, unsigned* storage);
		bool switchVal(bool Original, bool* storage);

	protected:
		bool active;
		OE_document* document;
};

class OE_actionsMeta : public OE_actions
{
	public:
		/** Default constructor */
		OE_actionsMeta(OE_document* doc);
		/** Default destructor */
		virtual ~OE_actionsMeta();
		virtual void undo();
		virtual void redo();
	protected:
		std::list<OE_actions*> actions;
};

