/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#pragma once

#include "actions/OE_actions.h"
#include "curves/OE_curve.h"
#include "curves/OE_joincurve.h"
#include "curves/OE_subcurve.h"
#include "curves/OE_pointcurve.h"

class OE_actionSelectionAddCurves : public OE_actions
{
	public:
		OE_actionSelectionAddCurves(OE_document* doc, std::list<OE_pointcurve*> curves, bool replace);

		virtual ~OE_actionSelectionAddCurves();

		virtual void undo();
		virtual void redo();

	protected:
		std::list<OE_pointcurve*> actionCurves;
};

class OE_actionSelectionRemoveCurves : public OE_actions
{
	public:
		OE_actionSelectionRemoveCurves(OE_document* doc, std::list<OE_pointcurve*> curves);

		virtual ~OE_actionSelectionRemoveCurves();

		virtual void undo();
		virtual void redo();

	protected:
		std::list<OE_pointcurve*> actionCurves;
};

class OE_actionSelectionClearCurves : public OE_actions
{
	public:
		OE_actionSelectionClearCurves(OE_document* doc);

		virtual ~OE_actionSelectionClearCurves();

		virtual void undo();
		virtual void redo();

	protected:
		std::list<OE_pointcurve*> actionCurves;
};

class OE_actionSelectionAddStitches : public OE_actions
{
    public:
		OE_actionSelectionAddStitches(OE_document* doc, std::list<OE_stitchs*> stitches, bool replace);

		virtual ~OE_actionSelectionAddStitches();

		virtual void undo();
		virtual void redo();

	protected:
		std::list<OE_stitchs*> actionStitches;
};


class OE_actionSelectionRemoveStitches : public OE_actions
{
	public:
		OE_actionSelectionRemoveStitches(OE_document* doc, std::list<OE_stitchs*> stitches);

		virtual ~OE_actionSelectionRemoveStitches();

		virtual void undo();
		virtual void redo();

	protected:
		std::list<OE_stitchs*> actionStitches;
};

class OE_actionSelectionClearStitches : public OE_actions
{
	public:
		OE_actionSelectionClearStitches(OE_document* doc);

		virtual ~OE_actionSelectionClearStitches();

		virtual void undo();
		virtual void redo();

	protected:
		std::list<OE_stitchs*> actionStitches;
};

class OE_actionSelectionClear : public OE_actions
{
	public:
		OE_actionSelectionClear(OE_document* doc);

		virtual ~OE_actionSelectionClear();

		virtual void undo();
		virtual void redo();

	protected:
		OE_actions* actionClearCurves;
		OE_actions* actionClearStitches;
};

class OE_actionMoveSelection : public OE_actions
{
	public:
		OE_actionMoveSelection(OE_document* doc, vector_2d offset);

		virtual ~OE_actionMoveSelection();

		virtual void undo();
		virtual void redo();

		void setMove(vector_2d offset);

	protected:
		vector_2d oldCoord;
		vector_2d offset;
};

class OE_actionScaleSelection : public OE_actions
{
	public:
		OE_actionScaleSelection(OE_document* doc, vector_2d ratio, vector_2d pos);

		virtual ~OE_actionScaleSelection();

		virtual void undo();
		virtual void redo();

		void setScale(vector_2d ratio);

	protected:
		vector_2d oldCoord;
		vector_2d ratio;
		vector_2d pos;
};

