/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#pragma once

#include "actions/OE_actions.h"
#include "stitchs/OE_stitchs.h"
#include "stitchs/OE_birailstitch.h"
#include "stitchs/OE_linestitch.h"
#include "stitchs/OE_linkstitch.h"

class OE_actionScaleSelectedLineStitchLen;
class OE_actionScaleSelectedBirailStitchLen;
class OE_actionScaleSelectedFillStitchLen;
class OE_actionScaleSelectedLineStitchWidth;
class OE_actionScaleSelectedFillStitchWidth;
class OE_actionSetSelectedLineStitchPattern;
class OE_actionSetSelectedFillStitchPattern;

class OE_actionNewStitch : public OE_actions
{
	public:
		OE_actionNewStitch(OE_document* doc, OE_stitchs* stitch);

		virtual ~OE_actionNewStitch();

		virtual void undo();
		virtual void redo();

	protected:
		OE_stitchs* actionStitch;
		OE_stitchs* linkStitch;
};

class OE_actionDelStitch : public OE_actions
{
	public:
		OE_actionDelStitch(OE_document* doc, OE_stitchs* stitch);

		virtual ~OE_actionDelStitch();

		virtual void undo();
		virtual void redo();

	protected:
		OE_stitchs* actionStitch;
		OE_linkstitch* linkStitchBefore;
		OE_linkstitch* linkStitchAfter;
		OE_stitchs* nextStitch;
		OE_stitchs* selNextLinkStitchBefore;
		OE_stitchs* selNextLinkStitchAfter;
		OE_stitchs* selNextStitch;

		OE_linkstitch* newLinkStitch;

		bool linkStitchBeforeSelected;
		bool linkStitchAfterSelected;
		bool selected;
};

class OE_actionSetStitchMaxLen : public OE_actions
{
	public:
		OE_actionSetStitchMaxLen(OE_document* doc, OE_stitchs* stitch, float maxLen);

		virtual ~OE_actionSetStitchMaxLen();

		virtual void undo();
		virtual void redo();

	protected:
		OE_stitchs* actionStitch;
		float oldMaxLen;
};

class OE_actionSetStitchThread : public OE_actions
{
	public:
		OE_actionSetStitchThread(OE_document* doc, OE_stitchs* stitch, OE_thread * thread);

		virtual ~OE_actionSetStitchThread();

		virtual void undo();
		virtual void redo();

	protected:
		OE_stitchs* actionStitch;
		OE_thread * oldThread;
};

class OE_actionScaleSelectedStitchLen : public OE_actions
{
	public:
		OE_actionScaleSelectedStitchLen(OE_document* doc, float scale);

		virtual ~OE_actionScaleSelectedStitchLen();

		virtual void undo();
		virtual void redo();

	protected:
		OE_actionScaleSelectedLineStitchLen* actionScaleSelectedLineStitchLen;
		OE_actionScaleSelectedBirailStitchLen* actionScaleSelectedBirailStitchLen;
		OE_actionScaleSelectedFillStitchLen* actionScaleSelectedFillStitchLen;
};

class OE_actionScaleSelectedStitchWidth : public OE_actions
{
	public:
	OE_actionScaleSelectedStitchWidth(OE_document* doc, float scale);

		virtual ~OE_actionScaleSelectedStitchWidth();

		virtual void undo();
		virtual void redo();

	protected:
		OE_actionScaleSelectedLineStitchWidth* actionScaleSelectedLineStitchWidth;
		OE_actionScaleSelectedFillStitchWidth* actionScaleSelectedFillStitchWidth;
};

class OE_actionSetSelectedStitchPattern : public OE_actions
{
	public:
		OE_actionSetSelectedStitchPattern(OE_document* doc, OE_pattern* pattern);

		virtual ~OE_actionSetSelectedStitchPattern();

		virtual void undo();
		virtual void redo();

	protected:
		OE_actionSetSelectedLineStitchPattern* actionSetSelectedLineStitchPattern;
		OE_actionSetSelectedFillStitchPattern* actionSetSelectedFillStitchPattern;
};


