/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#pragma once

#include "curves/OE_curve.h"
#include "curves/OE_joincurve.h"
#include "stitchs/OE_stitchs.h"

namespace Pakal { class Archive; }

class OE_birailstitch : public OE_stitchs
{
	public:
		//OE_birailstitch();
		OE_birailstitch(OE_thread* thread);
		OE_birailstitch(OE_thread* thread, bool reverse1, bool reverse2, float offset1, float offset2, float len);
		~OE_birailstitch();

		virtual void persist(Pakal::Archive* archive) override;

		virtual bool check();

		//virtual void refresh();

		virtual void refresh(float dpi, bool force = false);

		virtual void refreshDependency();


		void setReverse1(bool reverse);
		void setOffset1(float offset);
		void setReverse2(bool reverse);
		void setOffset2(float offset);
		void setLen(float len);

		OE_joincurve* getJoincurve1();
		bool getReverse1();
		float getOffset1();
		OE_joincurve* getJoincurve2();
		bool getReverse2();
		float getOffset2();
		float getLen();

		static float defaultLen;

	protected:
		OE_joincurve curve1;
		OE_joincurve curve2;
		bool reverse1;
		float offset1;
		bool reverse2;
		float offset2;
		float len;
};

