/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#include "OE_base.h"
#include "Archive.h"

OE_base::OE_base() : needRefresh(true)
{
}

OE_base::~OE_base()
{
	for (OE_base* dep : dependency)
	{
		dep->delDependency(this);
	}
}

void OE_base::persist(Pakal::Archive* archive)
{
	archive->refer("dependency", "object", dependency);
}

bool OE_base::getNeedRefresh()
{
	return needRefresh;
}

void OE_base::setNeedRefresh()
{
	needRefresh = true;
	for (OE_base* dep : dependency)
	{
		dep->setNeedRefresh();
	}
}

bool OE_base::ptInBounds(vector_2d pt, float* bounds)
{
	return pt.x >= bounds[0] && pt.x <= bounds[2] && pt.y >= bounds[1] && pt.y <= bounds[3];
}

void OE_base::addDependency(OE_base* object)
{
	dependency.insert(object);
}
void OE_base::removeDependency(OE_base* object)
{
	dependency.erase(object);
}

void OE_base::refreshDependency()
{
}

